//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  Lut.hpp -- template and derived ESC/I ESC/I-2 protocol commands
#pragma once

       
#include <utils/ESBuffer.h>
#include "ESCommonTypedef.h"
#include "ESImageInfo.h"
#include "ESCommonTypedef.h"
#include "ESImageInfo.h"
#include "EpsonScan2.h"
#include "ModelInfo.hpp"
#include "Filter.hpp"
#include "Utility.hpp"
namespace epsonscan
{
class Lut : public Filter
{
public:
           Lut(std::shared_ptr<KeyMgr> keyMgr,
                      std::shared_ptr<ModelInfo> modelInfo) : Filter(keyMgr, modelInfo)
            {
            }
            virtual void DoProcess(ESImageInfo& imageInfo, ES_CMN_FUNCS::BUFFER::CESHeapBuffer& inDataBuf) override
            {
     if (ES_IMAGE_INFO::GetESImageColorType(imageInfo) != ES_IMAGE_INFO::kESImageColorTypeMono)
     {
                        SDI_TRACE_LOG("backgroundRemoval Level = %d", keyMgr_->GetValueInt(kSDIBackgroundRemovalKey) );
                        if (keyMgr_->GetValueInt(kSDIBackgroundRemovalKey) != kSDIBackgroundRemovalNone && isFitPluginAvailable())
                        {
                            ProcessLUTUsingFit(imageInfo,
                                              inDataBuf,
                                              keyMgr_->GetValueInt(kSDIBrightnessKey),
                                             keyMgr_->GetValueInt(kSDIContrastKey),
                                             keyMgr_->GetValueInt(kSDIGammaKey),
                                             (SDIBackgroundRemoval)keyMgr_->GetValueInt(kSDIBackgroundRemovalKey),
                                             keyMgr_->GetWorkTempPath()
                                             );
                        }else{
                            ProcessLUT(imageInfo,inDataBuf, keyMgr_, modelInfo_);
                        }
     }
            }
            static bool isFitPluginAvailable();
protected:
            std::string Name() override
            {
                return "Lut";
            }
private:
 static void ProcessLUTUsingFit(ESImageInfo& imageInfo,
     ES_CMN_FUNCS::BUFFER::CESHeapBuffer& inDataBuf,
     ESNumber brightness,
     ESNumber contrast,
                    ESNumber gamma,
                    SDIBackgroundRemoval bgRemoval,
                    std::string workFolder
                    );
 static void ProcessLUT(ESImageInfo& imageInfo,
     ES_CMN_FUNCS::BUFFER::CESHeapBuffer& inDataBuf,
                    std::shared_ptr<KeyMgr> keyMgr,
                    std::shared_ptr<ModelInfo> modelInfo
                    );
      static void ProcessLUT(ESImageInfo& imageInfo,
     ES_CMN_FUNCS::BUFFER::CESHeapBuffer& inDataBuf,
                    unsigned char* gammaR,
                    unsigned char* gammaG,
                    unsigned char* gammaB
                    );
};
}
