//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  TransferCompression.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "TransferCompression.hpp"
#include "Scanner.hpp"
#include <ES2Command/ES2CommandAPI.h>
namespace epsonscan {
void TransferCompression::SetValue(const SDIInt& intVal)
{
 current_ = intVal;
}
void TransferCompression::Reset()
{
 current_ = SupportCompressionTransfer();
}
void TransferCompression::GetValue(SDIInt& intVal)
{
 intVal = current_;
}
void TransferCompression::GetCapability(SDICapability &capability)
{
 SetDefaultListCapability(capability);
 ESIndexSet availableTransferCompression;
 capability.supportLevel = kSDISupportLevelNone;
 if (SupportCompressionTransfer() && SupportRAWTransfer())
 {
  AddList(capability, kSDITransferCompressionJPEG);
  AddAllList(capability, kSDITransferCompressionJPEG);
  AddList(capability, kSDITransferCompressionRAW);
  AddAllList(capability, kSDITransferCompressionRAW);
  capability.supportLevel = kSDISupportLevelAvailable;
 }else{
  capability.supportLevel = kSDISupportLevelNone;
 }
}
ESDictionary TransferCompression::GetEngineKeyValues()
{
 auto dict = ESDictionary();
 if (SupportCompressionTransfer() && SupportRAWTransfer() == false)
 {
  dict[kESImageFormat] = (ESNumber)kESImageFormatJPEG;
 }
 if (SupportCompressionTransfer() == false && SupportRAWTransfer())
 {
  dict[kESImageFormat] = (ESNumber)kESImageFormatRaw;
 }
 return dict;
}
bool TransferCompression::SupportCompressionTransfer()
{
 ESIndexSet availableTransferCompression;
 Scanner* scanner = dataProvider_->GetScanner().get();
 if(scanner && scanner->GetAvailableValueForKey(kESImageFormat, availableTransferCompression))
 {
  if (availableTransferCompression.find(kESImageFormatJPEG) != availableTransferCompression.end())
     {
   return true;
  }
  else {
   return false;
  }
 }
 return false;
}
bool TransferCompression::SupportRAWTransfer()
{
 ESIndexSet availableTransferCompression;
 Scanner* scanner = dataProvider_->GetScanner().get();
 if(scanner && scanner->GetAvailableValueForKey(kESImageFormat, availableTransferCompression))
 {
  if (availableTransferCompression.find(kESImageFormatRaw) != availableTransferCompression.end())
     {
   return true;
  }
  else {
   return false;
  }
 }
 return false;
}
}
