//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  EpsonScan2.h -- template and derived ESC/I ESC/I-2 protocol commands
#pragma once

       
#ifndef _cplusplus
extern "C" {
#endif
#include <stdint.h>
#include <stdlib.h>
typedef struct SDIScannerDriver SDIScannerDriver;
typedef struct SDIDeviceFinder SDIDeviceFinter;
typedef struct SDIImage SDIImage;
typedef int32_t SDIInt;
typedef const char SDIChar;
#define MAX_MODEL_ID 20
#define MAX_IP_ADDR 64
#define MAX_DISPLAYNAME 50
typedef struct {
 SDIInt version;
 char modelID[MAX_MODEL_ID];
 SDIInt productID;
 char ipAddress[MAX_IP_ADDR];
 char displayName[MAX_DISPLAYNAME];
}SDIDeviceInfo;
typedef enum
{
  kSDITrasnferEventTypeImage,
  kSDITransferEventTypeComplete,
  kSDITransferEventTypeCancel,
  kSDITransferEventTypeStartContinuousScanInAFM,
  kSDITransferEventTypeStopContinuousScanInAFM
} SDITransferEventType;
typedef enum {
 kSDISupportLevelNone,
 kSDISupportLevelUnavailable,
 kSDISupportLevelAvailable
}SDISupportLevel;
typedef enum {
 kSDICapabilitTypeList,
    kSDICapabilitTypeRange
}SDICapabilityType;
#define SDI_CAPABILITY_LISTMAX 20
typedef struct {
 SDIInt version;
    SDISupportLevel supportLevel;
    SDICapabilityType capabilityType;
 SDIInt minValue;
 SDIInt maxValue;
 SDIInt allMinValue;
 SDIInt allMaxValue;
    SDIInt list[SDI_CAPABILITY_LISTMAX];
 SDIInt countOfList;
 SDIInt allList[SDI_CAPABILITY_LISTMAX];
 SDIInt countOfAllList;
}SDICapability;
typedef enum : SDIInt {
 kSDIErrorNone = 0,
 kSDIErrorUnknownError = 1,
 kSDIErrorInvalidParam = 2,
 kSDIErrorFileNotFound = 3,
 kSDIErrorDeviceNotOpened = 2000,
 kSDIErrorDeviceNotClosed = 2001,
 kSDIErrorDeviceInUse = 2002,
 kSDIErrorDeviceInBusy = 2003,
 kSDIErrorPaperEmpty = 2004,
 kSDIErrorPaperJam = 2005,
 kSDIErrorPaperDoubleFeed = 2006,
 kSDIErrorCoverOpen = 2007,
 kSDIErrorTrayClose = 2009,
 kSDIErrorCRLock = 2010,
 kSDIErrorLamp = 2011,
 kSDIErrorLampTime = 2012,
 kSDIErrorAuthFailure = 2013,
 kSDIErrorNoPermission = 2014,
 kSDIErrorLowBattery = 2015,
 kSDIErrorDataSend = 2016,
 kSDIErrorDataReceive = 2017,
 kSDIErrorInvalidResponse = 2019,
 kSDIErrorDeviceOpen = 2020,
 kSDIErrorDeviceIncorrect = 2021,
 kSDIErrorConnectionTimeout = 2022,
 kSDIErrorDeviceDisconnected = 2023,
 kSDIErrorDeviceNotFound = 2024,
 kSDIErrorDeviceFatalError = 2025,
 kSDIErrorServerError = 2026,
 kSDIErrorCRSheetSensorError = 2027,
 kSDIErrorPaperProtect = 2028,
 kSDIErrorGapSensorError = 2029,
 kSDIErrorDeviceFormUnstable = 2030,
 kSDIErrorDeviceFormChangedInterruptedly = 2031,
 kSDIErrorSepLeverChangedInterruptedly = 2032,
 kSDIErrorUnscannableDeviceConfig1 = 2033,
 kSDIErrorPaperDoubleFeed2 = 2035,
 kSDIErrorETSensorError = 2036,
 kSDIErrorDiskFull = 3000,
 kSDIErrorWriteFile = 3002,
 kSDIErrorWriteTempFile = 3003,
 kSDIErrorFileNameConflict = 3004,
 kSDIErrorCreateWorkDir = 3005,
 kSDIErrorNoMemory = 4001,
 kSDIErrorLongPaperUnsupporetedResolution = 5007,
 kSDIErrorScanAreaTooLarge = 5009,
 kSDIErrorScanAreaTooSmall = 5010,
 kSDIErrorDocumentSeparationLever = 5011,
 kSDIErrorCardLever = 5012,
 kSDIErrorRotateUnsupportedResolution = 5013,
 kSDIErrorJsonParse = 11001,
 kSDIErrorValueType = 11004,
 kSDIErrorModuleNotFound = 11003,
} SDIError;
typedef enum : SDIInt
{
  kSDIInterruptEventTypePushScan,
 kSDIInterruptEventTypeDisconnect,
 kSDIInterruptEventTypeGlassDirty,
 kSDIInterruptEventTypeCleaningRequired
} SDIInterruptEventType;
typedef enum : SDIInt
{
 kSDIValueTypeInt,
 kSDIValueTypeString
} SDIValueType;
typedef enum : SDIInt
{
  kSDIOperationTypeNew,
  kSDIOperationTypeCancel,
  kSDIOperationTypeStartAFM,
  kSDIOperationTypeStopAFM
} SDIOperationType;
typedef void (*DeviceFinderCallBackProc)(SDIDeviceFinder* finder, const SDIDeviceInfo* info, void* userdata);
typedef void (*InterruptEventCallBackProc)(SDIScannerDriver* driver, SDIInterruptEventType event , void* userdata);
typedef SDIError (*SDIDeviceFinder_CreatePtr)(SDIDeviceFinter** outFinder);
typedef SDIError (*SDIDeviceFinder_StartDiscoveryPtr)(SDIDeviceFinder* finder,DeviceFinderCallBackProc callBack, void* param);
typedef SDIError (*SDIDeviceFinder_StopDiscoveryPtr)(SDIDeviceFinder* finder);
typedef SDIError (*SDIDeviceFinder_SetTimeoutPtr)(SDIDeviceFinder* finder, SDIInt seconds);
typedef SDIError (*SDIDeviceFinder_GetDevicesPtr)(SDIDeviceFinder* finder, SDIDeviceInfo** outDevices, SDIInt* outCount);
typedef SDIError (*SDIDeviceFinder_FreeDevicesPtr)(SDIDeviceInfo* devices);
typedef SDIError (*SDIDeviceFinder_DisposePtr)(SDIDeviceFinder* finder);
typedef SDIError (*SDIScannerDriver_CreatePtr)(SDIScannerDriver** driver,
              const SDIDeviceInfo* deviceInfo,
                 InterruptEventCallBackProc callBack,
                 void *userdata);
typedef SDIError (*SDIScannerDriver_OpenPtr)(SDIScannerDriver* driver);
typedef SDIError (*SDIScannerDriver_ResetPtr)(SDIScannerDriver* driver);
typedef SDIError (*SDIScannerDriver_GetAllKeysPtr)(SDIScannerDriver* driver, SDIChar **outKeys, SDIInt* outCount);
typedef SDIError (*SDIScannerDriver_SetValuePtr)(SDIScannerDriver* driver, SDIChar* key, SDIValueType valType, void* value, SDIInt size);
typedef SDIError (*SDIScannerDriver_GetValuePtr)(SDIScannerDriver* driver, SDIChar* key, SDIValueType valType, void* value, SDIInt size);
typedef SDIError (*SDIScannerDriver_GetDefaultPtr)(SDIScannerDriver* driver, SDIChar* key, SDIValueType valType, void* value);
typedef SDIError (*SDIScannerDriver_GetCapabilityPtr)(SDIScannerDriver* driver, SDIChar* key, SDICapability* outCapability);
typedef SDIError (*SDIScannerDriver_DoScanJobPtr)(SDIScannerDriver* driver,
                SDIOperationType operationType);
typedef bool (*SDIScannerDriver_CheckNextTransferEventPtr)(SDIScannerDriver* driver ,bool waitUntilReceived);
typedef SDIError (*SDIScannerDriver_GetNextTransferEventPtr)(SDIScannerDriver* driver,
                 SDITransferEventType* outType,
                  SDIImage* outImageData,
                     SDIError* outError);
typedef SDIError (*SDIScannerDriver_ClosePtr)(SDIScannerDriver* driver);
typedef SDIError (*SDIScannerDriver_DisposePtr)(SDIScannerDriver* driver);
typedef SDIError (*SDIImage_CreatePtr)(SDIImage** image);
typedef SDIInt (*SDIImage_GetWidthPtr)(SDIImage* image);
typedef SDIInt (*SDIImage_GetHeightPtr)(SDIImage* image);
typedef SDIInt (*SDIImage_GetSamplesPerPixelPtr)(SDIImage* image);
typedef SDIInt (*SDIImage_GetBitsPerSamplePtr)(SDIImage* image);
typedef SDIInt (*SDIImage_GetPathPtr)(SDIImage* image, SDIChar* buf, SDIInt bufSize);
typedef SDIInt (*SDIImage_DisposePtr)(SDIImage* image);
typedef bool (*SDIDeviceInfo_ResolvePtr)(SDIDeviceInfo* resolveDevice);
static SDIChar* kSDIFunctionalUnitKey = "FunctionalUnit";
typedef enum : SDIInt
{
  kSDIFunctionalUnitFlatbed = 0,
  kSDIFunctionalUnitDocumentFeeder = 1,
  kSDIFunctionalUnitCarrierSheet = 2
}SDIFunctionalUnit;
static SDIChar* kSDIGuidePositionKey = "GuidePosition";
typedef enum : SDIInt
{
  kSDIGuidePositionLeft = 0,
  kSDIGuidePositionCenter = 1,
  kSDIGuidePositionRight = 2
}SDIGuidePosition;
static SDIChar* kSDIColorTypeKey = "ColorType";
typedef enum : SDIInt
{
        kSDIColorTypeRGB24 = 0,
        kSDIColorTypeMono8 = 1,
        kSDIColorTypeMono1 = 2,
        kSDIColorTypeRGB48 = 3,
        kSDIColorTypeMono16 = 4
}SDIColorType;
static SDIChar* kSDIResolutionKey = "Resolution";
static SDIChar* kSDIScanAreaWidthKey = "ScanAreaWidth";
static SDIChar* kSDIScanAreaHeightKey = "ScanAreaHeight";
static SDIChar* kSDIScanAreaOffsetXKey = "ScanAreaOffsetX";
static SDIChar* kSDIScanAreaOffsetYKey = "ScanAreaOffsetY";
static SDIChar* kSDIMaxScanAreaWidthKey = "ScanAreaMaxWidth";
static SDIChar* kSDIMaxScanAreaHeightKey = "ScanAreaMaxHeight";
static SDIChar* kSDIMaxScanAreaHeightInLongPaperKey = "ScanAreaMaxHeightInLongPaper";
static SDIChar* kSDIDropoutColorKey = "DropoutColor";
typedef enum : SDIInt
{
 kSDIDropoutColorNone = 0,
 kSDIDropoutColorRed = 1,
 kSDIDropoutColorGreen = 2,
 kSDIDropoutColorBlue = 3
} SDIDropoutColor;
static SDIChar* kSDIOrientationKey = "Orientation";
typedef enum : SDIInt
{
 kSDIOrientationNone = 0,
 kSDIOrientation90 = 1,
 kSDIOrientation180 = 2,
 kSDIOrientation270 = 3,
 kSDIOrientationAuto = 4
} SDIOrientation;
static SDIChar* kSDIGammaKey = "Gamma";
static SDIChar* kSDIImageFormatKey = "ImageFormat";
typedef enum : SDIInt
{
  kSDIImageFormatRaw,
  kSDIImageFormatJPEG,
  kSDIImageFormatTIFF,
  kSDIImageFormatPDF,
  kSDIImageFormatPNG,
  kSDIImageFormatPNM,
  kSDIImageFormatColorJpegMonoPNM,
  kSDIImageFormatBMP
}SDIImageFormat;
static SDIChar* kSDIMultiTiffEnabledKey = "MultiTiffEnabled";
static SDIChar* kSDITiffCompressionKey = "MultiTiffCompression";
typedef enum : SDIInt
{
 kSDITiffCompressionNone,
 kSDITiffCompressionCCITFAX4
} SDITiffCompression;
static SDIChar* kSDIThresholdKey = "Threshold";
static SDIChar* kSDIBrightnessKey = "Brightness";
static SDIChar* kSDIContrastKey = "Contrast";
static SDIChar* kSDIJpegQuality = "JpegQuality";
static SDIChar* kSDIJpegProgressiveKey = "jpegProgressive";
static SDIChar* kSDIDuplexTypeKey = "DuplexType";
static SDIChar* kSDIPaperLoadedKey = "PaperLoaded";
static SDIChar* kSDIPagesTobeScannedKey = "PagesTobeScanned";
static SDIChar* kSDIDoubleFeedDetectionKey = "DoubleFeedDetection";
static SDIChar* kSDIDoubleFeedDetectionAreaMinKey = "DoubleFeedDetectionAreaMin";
static SDIChar* kSDIDoubleFeedDetectionAreaLengthKey = "DoubleFeedDetectionAreaLength";
static SDIChar* kSDIDoubleFeedDetectionLevelKey = "DoubleFeedDetectionLevel";
typedef enum : SDIInt
{
  kSDIDoubleFeedDetectionLow = 0,
  kSDIDoubleFeedDetectionHigh = 1
} SDIDoubleFeedDetectionLevel;
static SDIChar* kSDIBlankPageSkipKey = "BlankPageSkip";
static SDIChar* kSDIBlankPageSkipLevelKey = "BlankPageSkipLevel";
static SDIChar* kSDIPaperDeskewKey = "PaperDeskew";
static SDIChar* kSDIBackgroundRemovalKey = "BackgroundRemoval";
typedef enum : SDIInt
{
   kSDIBackgroundRemovalNone = 0,
   kSDIBackgroundRemovalNormal = 1,
   kSDIBackgroundRemovalHigh = 2
}SDIBackgroundRemoval;
static SDIChar* kSDIAutoSizeKey = "AutoSize";
typedef enum : SDIInt
{
 kSDIAutoSizeNone = 0,
 kSDIAutoSizeStandard = 1,
 kSDIAutoSizeLongPaper = 2
}SDIAutoSizeType;
static SDIChar* kSDITransferCompressionKey = "TransferCompression";
typedef enum : SDIInt
{
 kSDITransferCompressionRAW = 0,
 kSDITransferCompressionJPEG = 1,
}SDITransferCompression;
static SDIChar* kSDISleepTimeKey = "SleepTime";
 static SDIChar* kSDIAutoPowerOffTimeKey = "AutoPowerOffTime";
 static SDIChar* kSDIAutoPowerOffTimeBatteryKey = "AutoPowerOffTimeBattery";
static SDIChar* kSDIScanCounterKey = "ScanCounterKey";
static SDIChar* kSDIADFScanCounterKey = "ADFScanCounterKey";
static SDIChar* kSDIADFDuplexCounterKey = "ADFDuplexScanCounterKey";
static SDIChar* kSDIADFCarrierSheetCounterKey = "ADFCarrierSheetScanCounterKey";
static SDIChar* kSDICardScanCounterKey = "CardScanCounterKey";
static SDIChar* kSDIPaperJamCounterKey = "PaperJamCounter";
static SDIChar* kSDIDoubleFeedCounterKey = "DoubleFeedCounter";
static SDIChar* kSDIPickupRollerCounterKey = "PickupRollerCounter";
static SDIChar* kSDIPaperEndDetectionKey = "PaperEndDetection";
static SDIChar* kSDIADFHeightMinKey = "ADFHeightMin";
static SDIChar* kSDIADFDuplexHeightMinKey = "ADFDuplexHeightMin";
static SDIChar* kSDIADFWidthMinKey = "ADFWidthMin";
static SDIChar* kSDI2in1ModeKey = "2in1Mode";
static SDIChar* kSDIScannerPositionKey = "ScannerPositon";
 typedef enum : SDIInt
{
 kSDIScannerPositionFlat = 0,
 kSDIScannerPositionTilt = 1,
 kSDIScannerPositionInvalid = 2
}SDIScannerPosition;
static SDIChar* kSDIUnscannebleScanParameterStatusAllKey = "UnscannebleScanParameter";
static SDIChar* kSDIBackgroundColorKey = "BackgroundColor";
 typedef enum : SDIInt
{
 kSDISDIBackgroundColorWhite = 0,
 kSDISDIBackgroundColorBlack = 1,
 kSDISDIBackgroundColorGray = 2,
 kSDISDIBackgroundColorNone = 3
}SDIBackgroundColor;
static SDIChar* kSDIAFMModeKey = "AFMMode";
static SDIChar* kSDIRollerKitCounterKey = "RollerKitCounter";
static SDIChar* kSDIRollerKitLifeLimitKey = "RollerKitLifeLimit";
static SDIChar* kSDIRollerKitNearEndKey = "RollerKitNearEnd";
static SDIChar* kSDIRollerCounterKey = "RollerCounter";
static SDIChar* kSDIRollerLifeLimitKey = "RollerLifeLimit";
static SDIChar* kSDIRollerNearEndKey = "RollerNearEnd";
static SDIChar* kSDIRetardRollerCounterKey = "RetardRollerCounter";
static SDIChar* kSDIRetardRollerLifeLimitKey = "RetardRollerLifeLimit";
static SDIChar* kSDIRetardRollerNearEndKey = "RetardRollerNearEnd";
static SDIChar* kSDIRestrictFunctionKey = "RestrictFunction";
static SDIChar* kSDIAFMTimeoutKey = "AFMTimeout";
static SDIChar* kSDIErrorStatusKey = "ErrorStatus";
SDIError SDIDeviceFinder_Create(SDIDeviceFinter** outFinder);
SDIError SDIDeviceFinder_StartDiscovery(SDIDeviceFinder* finder,DeviceFinderCallBackProc callBack, void* userdata);
SDIError SDIDeviceFinder_StopDiscovery(SDIDeviceFinder* finder);
SDIError SDIDeviceFinder_SetTimeout(SDIDeviceFinder* finder, SDIInt seconds);
SDIError SDIDeviceFinder_GetDevices(SDIDeviceFinder* finder, SDIDeviceInfo** outDevices, SDIInt* outCount);
SDIError SDIDeviceFinder_Dispose(SDIDeviceFinder* finder);
bool SDIDeviceInfo_Resolve(SDIDeviceInfo* resolveDevice);
SDIError SDIScannerDriver_Create(SDIScannerDriver** driver,
          const SDIDeviceInfo* deviceInfo,
            InterruptEventCallBackProc callBack,
           void *userdata);
SDIError SDIScannerDriver_Open(SDIScannerDriver* driver);
SDIError SDIScannerDriver_Reset(SDIScannerDriver* driver);
SDIError SDIScannerDriver_GetAllKeys(SDIScannerDriver* driver, SDIChar **outKeys, SDIInt* outCount);
SDIError SDIScannerDriver_SetValue(SDIScannerDriver* driver, SDIChar* key, SDIValueType valType, void* value, SDIInt size);
SDIError SDIScannerDriver_GetValue(SDIScannerDriver* driver, SDIChar* key, SDIValueType valType, void* value, SDIInt size);
SDIError SDIScannerDriver_GetDefault(SDIScannerDriver* driver, SDIChar* key, SDIValueType valType, void* value);
SDIError SDIScannerDriver_GetCapability(SDIScannerDriver* driver, SDIChar* key, SDICapability* outCapability);
SDIError SDIScannerDrive_DoScanJob(SDIScannerDriver* driver,
           SDIOperationType operationType);
bool SDIScannerDriver_CheckNextTransferEvent(SDIScannerDriver* driver, bool waitUntilReceived);
SDIError SDIScannerDriver_GetNextTransferEvent(SDIScannerDriver* driver,
              SDITransferEventType* outType,
              SDIImage* outImageData,
              SDIError* outError
             );
SDIError SDIScannerDriver_Close(SDIScannerDriver* driver);
SDIError SDIScannerDriver_Dispose(SDIScannerDriver* driver);
SDIError SDIImage_Create(SDIImage** image);
SDIInt SDIImage_GetWidth(SDIImage* image);
SDIInt SDIImage_GetHeight(SDIImage* image);
SDIInt SDIImage_GetSamplesPerPixel(SDIImage* image);
SDIInt SDIImage_GetBitsPerSample(SDIImage* image);
SDIInt SDIImage_GetPath(SDIImage* image, int8_t * buf, SDIInt bufSize);
typedef bool (*SDIImage_CheckImageIsBlankPtr)(SDIImage* image);
SDIInt SDIImage_CheckImageIsBlank(SDIImage* image);
SDIInt SDIImage_Dispose(SDIImage* image);
typedef bool (*SDIMultipage_StartPtr)(SDIScannerDriver* driver, const char* destPath, SDIImageFormat format);
typedef bool (*SDIMultipage_AddPtr)(SDIScannerDriver* driver, const char* sourcePath);
typedef bool (*SDIMultipage_FinPtr)(SDIScannerDriver* driver);
bool SDIMultipage_Start(SDIScannerDriver* driver, const char* destPath, SDIImageFormat format);
bool SDIMultipage_Add(SDIScannerDriver* driver, const char* sourcePath);
bool SDIMultipage_Fin(SDIScannerDriver* driver);
#ifndef _cplusplus
}
#endif
