//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  cfg-obj.h -- template and derived ESC/I ESC/I-2 protocol commands

#ifndef included_cfg_obj_h
#define included_cfg_obj_h 
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdint.h>
#include <sane/sane.h>
#include "list.h"
#ifdef __cplusplus
extern "C"
{
#endif
  extern const char *cfg_file_name;
  typedef enum
    {
      CFG_KEY_NET = 0,
      CFG_KEY_PIO,
      CFG_KEY_SCSI,
      CFG_KEY_USB,
      CFG_KEY_INTERPRETER,
      CFG_KEY_FS_BLACKLIST,
      CFG_KEY_OPTION,
      _CFG_KEY_ID_TERMINATOR_
    }
    cfg_key_id_type;
  typedef const char * cfg_key_type;
  typedef struct
  {
    const char *spec;
  } cfg_net_info;
  typedef struct
  {
    const char *vendor;
    const char *model;
  } cfg_scsi_info;
  typedef struct
  {
    uint16_t vendor;
    uint16_t product;
  } cfg_usb_info;
  typedef struct
  {
    uint16_t vendor;
    uint16_t product;
    const char *library;
    const char *firmware;
  } cfg_interpreter_info;
  void * cfg_init (const char *pkgdatadir, SANE_Status *status);
  void * cfg_exit (void *self);
  void cfg_find (const void *self, cfg_key_id_type id, list *dev_list);
  list * cfg_seen (const void *self, cfg_key_id_type id);
  bool cfg_has (const void *self, cfg_key_id_type id);
  void cfg_set (void *self, cfg_key_id_type id, bool value);
  bool cfg_has_value (const void *self, cfg_key_id_type id, const char* val);
  cfg_key_type cfg_key (const void *self, cfg_key_id_type id);
#ifdef __cplusplus
}
#endif
#endif
