//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  message.h -- template and derived ESC/I ESC/I-2 protocol commands

#ifndef message_h
#define message_h 
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#ifndef ENABLE_DEBUG
#define ENABLE_DEBUG 1
#endif
#ifdef __cplusplus
extern "C"
{
#endif
  typedef enum
    {
      ERR_FATAL = (1 << 0),
      ERR_MAJOR = (1 << 1),
      ERR_MINOR = (1 << 2),
      LOG_INFO = (1 << 3),
      LOG_CALL = (1 << 4),
      LOG_DATA = (1 << 5),
      DBG_CMD = (1 << 6),
      DBG_HEX = (1 << 7),
      DBG_IMG = (1 << 8),
    }
    msg_level_type;
  extern unsigned long msg_level;
#define MSG_DBG_IMG_THRESHOLD 512
#define MSG_MODULE "epsonscan2"
#define FMT_FILE "%d: "
#define FMT_LINE , __LINE__
#define FMT_MODULE FMT_FILE "[%s]"
#define FMT_FATAL FMT_MODULE "[F] "
#define FMT_MAJOR FMT_MODULE "[M] "
#define FMT_MINOR FMT_MODULE "[m] "
#define FMT_INFO FMT_MODULE "{I} "
#define FMT_CALL FMT_MODULE "{C} " "%s "
#define FMT_DATA FMT_MODULE "{D} "
#define FMT_CMD FMT_MODULE "(e) "
#define FMT_HEX "[" MSG_MODULE "]" "(x) "
#define FMT_IMG "[" MSG_MODULE "]" "(i) "
#if ENABLE_DEBUG
#define err_fatal(fmt,arg...) \
  do \
    { \
      if (ERR_FATAL <= msg_level) \
        fprintf (stderr, FMT_FATAL fmt "\n" FMT_LINE, MSG_MODULE, \
                 ## arg); \
    } \
  while (0)
#define err_major(fmt,arg...) \
  do \
    { \
      if (ERR_MAJOR <= msg_level) \
        fprintf (stderr, FMT_MAJOR fmt "\n" FMT_LINE, MSG_MODULE, \
                 ## arg); \
    } \
  while (0)
#define err_minor(fmt,arg...) \
  do \
    { \
      if (ERR_MINOR <= msg_level) \
        fprintf (stderr, FMT_MINOR fmt "\n" FMT_LINE, MSG_MODULE, \
                 ## arg); \
    } \
  while (0)
#define log_info(fmt,arg...) \
  do \
    { \
      if (LOG_INFO <= msg_level) \
        fprintf (stderr, FMT_INFO fmt "\n" FMT_LINE, MSG_MODULE, \
                 ## arg); \
    } \
  while (0)
#define log_call(fmt,arg...) \
  do \
    { \
      if (LOG_CALL <= msg_level) \
        fprintf (stderr, FMT_CALL fmt "\n" FMT_LINE, MSG_MODULE, \
                 __func__, ## arg); \
    } \
  while (0)
#define log_data(fmt,arg...) \
  do \
    { \
      if (LOG_DATA <= msg_level) \
        fprintf (stderr, FMT_DATA fmt "\n" FMT_LINE, MSG_MODULE, \
                 ## arg); \
    } \
  while (0)
#define dbg_cmd(buf,sz) \
  do \
    { \
      if (DBG_CMD <= msg_level) \
        fprintf (stderr, FMT_CMD fmt "\n" FMT_LINE, MSG_MODULE, \
                 ## arg); \
    } \
  while (0)
#define dbg_hex(buf,sz) \
  do \
    { \
      if (DBG_HEX <= msg_level) \
        msg_dump (FMT_HEX, buf, sz); \
    } \
  while (0)
#define dbg_img(buf,sz) \
  do \
    { \
      if (DBG_IMG <= msg_level) \
        msg_dump (FMT_IMG, buf, sz); \
    } \
  while (0)
  void msg_init (void);
  void msg_dump (const char *, const void *, size_t);
#else
#define err_fatal(fmt,arg...) 
#define err_major(fmt,arg...) 
#define err_minor(fmt,arg...) 
#define log_info(fmt,arg...) 
#define log_call(fmt,arg...) 
#define log_data(fmt,arg...) 
#define dbg_cmd(buf,sz) 
#define dbg_hex(buf,sz) 
#define dbg_img(buf,sz) 
#define msg_init() 
#define msg_dump(fmt,buf,sz) 
#endif
#ifdef __cplusplus
}
#endif
#endif
