//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  aboutepsonscan2dialog.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "aboutepsonscan2dialog.h"
#include "ui_aboutepsonscan2dialog.h"
#include "supervisor.h"
#include "../Include/Config.h"
#include <sstream>
#include "PathUtils.h"
AboutEpsonScan2Dialog::AboutEpsonScan2Dialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AboutEpsonScan2Dialog)
{
    ui->setupUi(this);
    this->setFixedSize(this->size());
    ui->Model_List_tableWidget->setColumnWidth(0, ui->Model_List_tableWidget->width()*0.7);
    ui->Model_List_tableWidget->setColumnWidth(1, ui->Model_List_tableWidget->width()*0.3);
#ifdef NOUSEQRC
    std::string strIconPath = EPSON_INSTALL_PATH;
    ES_CMN_FUNCS::PATH::ES_CombinePath(strIconPath, strIconPath, "Resources/Icons/escan2_app.ico");
    ui->Help_toolButton->setIcon(QIcon(strIconPath.c_str()));
#endif
}
AboutEpsonScan2Dialog::~AboutEpsonScan2Dialog()
{
    delete ui;
}
void AboutEpsonScan2Dialog::closeEvent(QCloseEvent* event){
}
void AboutEpsonScan2Dialog::Initialize()
{
    std::string dir1(COMMON_SHARE_PATH);
    ES_CMN_FUNCS::PATH::ES_CombinePath(dir1, dir1, "doc/");
    this->ReadVersionFile(dir1.c_str());
    ui->Locale_label->setText(QLocale::system().name());
}
#define VERSION_LENGTH 9
void AboutEpsonScan2Dialog::ReadVersionFile(const char* version_path)
{
    std::string dir_name;
    std::string package_name;
    std::string version;
    auto files = ES_CMN_FUNCS::PATH::EnumFilesInDir("", version_path);
 for(auto file : files)
    {
        std::string filename;
        ES_CMN_FUNCS::PATH::ES_GetFileName(filename, file, FALSE);
        if(strncmp(filename.c_str(), "epsonscan2-", strlen("epsonscan2-")) == 0){
            dir_name = filename.c_str();
            const size_t strLen = filename.length();
            version = filename.substr(strLen - VERSION_LENGTH, VERSION_LENGTH);
            package_name = dir_name;
            int del_count = version.size() + 1;
            while(del_count > 0){
                package_name.erase(package_name.end() - del_count);
                del_count--;
            }
            QString temp = package_name.c_str();
            ui->Model_List_tableWidget->insertRow(0);
            ui->Model_List_tableWidget->setItem( 0, 0, new QTableWidgetItem( temp ));
            ui->Model_List_tableWidget->setCurrentItem(ui->Model_List_tableWidget->item(0, 0));
            temp = version.c_str();
            ui->Model_List_tableWidget->setItem( 0, 1, new QTableWidgetItem( temp ));
        }
    }
}
