//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  lastusedsettings.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include <fstream>
#define BOOST_NO_CXX11_RVALUE_REFERENCES 1
#include <boost/any.hpp>
#include <utils/PathUtils.h>
#include <QStandardPaths>
#include "devicelist.h"
#include "lastusedsettings.h"
#include "../Include/Config.h"
LastUsedSettings::LastUsedSettings()
{
   std::string dir1(qPrintable(QStandardPaths::writableLocation(QStandardPaths::HomeLocation)));
   ES_CMN_FUNCS::PATH::ES_CombinePath(dir1, dir1,".epsonscan2/Connection");
   ES_CMN_FUNCS::PATH::ES_CombinePath(dir1, dir1,"PreferredInfo.dat");
   this->PreferredInfoFilePath = dir1.c_str();
   std::string dir3(qPrintable(QStandardPaths::writableLocation(QStandardPaths::HomeLocation)));
   ES_CMN_FUNCS::PATH::ES_CombinePath(dir3, dir3,".epsonscan2/Settings/");
   this->SettingsFilePath = dir3.c_str();
}
bool LastUsedSettings::CheckLastUsedDevice()
{
    std::string dir1(qPrintable(QStandardPaths::writableLocation(QStandardPaths::HomeLocation)));
    ES_CMN_FUNCS::PATH::ES_CombinePath(dir1, dir1,".epsonscan2/");
    if(!(bool)ES_CMN_FUNCS::PATH::ES_IsExistFile(dir1)){
        ES_CMN_FUNCS::PATH::ES_MakeFolder(dir1);
    }
    std::string folder;
    ES_CMN_FUNCS::PATH::ES_GetFolderPath(folder, this->PreferredInfoFilePath);
    if(ES_CMN_FUNCS::PATH::ES_IsExistFolder(folder) == FALSE)
    {
        ES_CMN_FUNCS::PATH::ES_MakeFolder(folder);
        return false;
    }
    const std::string connect_path(this->PreferredInfoFilePath.c_str());
    return (bool)ES_CMN_FUNCS::PATH::ES_IsExistFile(connect_path);
}
bool LastUsedSettings::ReadPreferredInfo()
{
  ESDictionary out;
  if(!this->CheckLastUsedDevice()){
      return false;
  }
  ES_CMN_FUNCS::JSON::JSONFiletoDictionary(this->PreferredInfoFilePath.c_str(), out);
  ESString connection = boost::any_cast<ESString>(out["Connection"]);
  ESString model_id = boost::any_cast<ESString>(out["ESModelID"]);
  if(connection == "USB"){
      if(!this->SelectUSBDevice(model_id)){
          return false;
      }
  }else {
      if(!this->SelectNetworkDevice(connection)){
          return false;
      }
  }
  return true;
}
bool LastUsedSettings::SelectUSBDevice(std::string model_id)
{
    auto device = DeviceList::device_list.begin();
    int i = 0;
    while(i < (signed)DeviceList::device_list.size()){
        if(device->modelID == model_id){
            DeviceList::select_device = i;
            return true;
        }
        ++device;
        i++;
    }
    return false;
}
bool LastUsedSettings::SelectNetworkDevice(std::string ip)
{
    auto device = DeviceList::device_list.begin();
    int i = 0;
    while(i < (signed)DeviceList::device_list.size()){
        if(device->ipAddress == ip){
            DeviceList::select_device = i;
            return true;
        }
        ++device;
        i++;
    }
    device = DeviceList::manu_network_device_list.begin();
    i = DeviceList::device_list.size();
    while(i < (signed)DeviceList::manu_network_device_list.size()){
        if(device->ipAddress == ip){
            DeviceList::select_device = i;
            return true;
        }
        ++device;
        i++;
    }
    return false;
}
void LastUsedSettings::CreatePreferredInfo(SDIDeviceInfo dev_info)
{
    if(this->CheckLastUsedDevice()){
        try {
            DeleteFile(this->PreferredInfoFilePath.c_str());
        }
        catch (...) {
            throw;
        }
    }
    ESDictionary dict;
    dict["ESModelID"] = (ESString)dev_info.modelID;
    if(dev_info.ipAddress[0] == '\0'){
        dict["Connection"] = (ESString)"USB";
    }else {
        dict["Connection"] = (ESString)dev_info.ipAddress;
    }
    ES_CMN_FUNCS::JSON::DictionaryToJSONFile(dict, this->PreferredInfoFilePath.c_str());
}
void LastUsedSettings::DeletePreferredInfo(void)
{
    try {
        DeleteFile(this->PreferredInfoFilePath.c_str());
    }
    catch (...) {
        throw;
    }
}
bool LastUsedSettings::CheckCommonSettingsFile()
{
    std::string dir1(qPrintable(QStandardPaths::writableLocation(QStandardPaths::HomeLocation)));
    ES_CMN_FUNCS::PATH::ES_CombinePath(dir1, dir1,".epsonscan2/");
    if(!(bool)ES_CMN_FUNCS::PATH::ES_IsExistFile(dir1)){
        ES_CMN_FUNCS::PATH::ES_MakeFolder(dir1);
    }
    if(ES_CMN_FUNCS::PATH::ES_IsExistFolder(this->SettingsFilePath) == FALSE)
    {
        ES_CMN_FUNCS::PATH::ES_MakeFolder(this->SettingsFilePath);
    }
    std::string common_settings_file = this->SettingsFilePath.c_str();
    ES_CMN_FUNCS::PATH::ES_CombinePath(common_settings_file, common_settings_file, "CommonSettings.SF2");
    return ES_CMN_FUNCS::PATH::ES_IsExistFile(common_settings_file);
}
CONFIGURATION LastUsedSettings::ReadCommonSettingsFile()
{
    CONFIGURATION config_val;
    memset(&config_val, 0, sizeof(CONFIGURATION));
    if(this->CheckCommonSettingsFile()){
        std::string common_settings_file = this->SettingsFilePath.c_str();
        common_settings_file = common_settings_file + "CommonSettings.SF2";
        ESDictionary out;
        ES_CMN_FUNCS::JSON::JSONFiletoDictionary(common_settings_file, out);
        config_val.units = (Units)boost::any_cast<ESNumber>(out["ESUnit"]);
        config_val.last_used_settings = boost::any_cast<ESNumber>(out["ESLastUsedSettings"]);
        config_val.compression = boost::any_cast<ESNumber>(out["ESCompression"]);
        config_val.open_img_folder = boost::any_cast<ESNumber>(out["ESOpenFolder"]);
        config_val.sleep_timer = boost::any_cast<ESNumber>(out["ESSleepTimer"]);
        config_val.ac_power = (Time)boost::any_cast<ESNumber>(out["ESACPower"]);
        config_val.usb_power = (Time)boost::any_cast<ESNumber>(out["ESUSBPower"]);
    }else {
        config_val.units = (Units)unit_inchi;
        config_val.last_used_settings = true;
        config_val.compression = true;
        config_val.open_img_folder = true;
        config_val.sleep_timer = 240;
        config_val.ac_power = t_Off;
        config_val.usb_power = t_Off;
    }
    return config_val;
}
CONFIGURATION LastUsedSettings::ReadMainWindowInfo()
{
    Configuration& config = Configuration::GetConfiguration();
    if(this->CheckCommonSettingsFile()){
        std::string common_settings_file = this->SettingsFilePath.c_str();
        common_settings_file = common_settings_file + "CommonSettings.SF2";
        ESDictionary out;
        ES_CMN_FUNCS::JSON::JSONFiletoDictionary(common_settings_file, out);
        config.config_para.WindowPositionX = boost::any_cast<ESNumber>(out["WindowPositionX"]);
        config.config_para.WindowPositionY = boost::any_cast<ESNumber>(out["WindowPositionY"]);
        config.config_para.WindowWidth = boost::any_cast<ESNumber>(out["WindowWidth"]);
        config.config_para.WindowHeight = boost::any_cast<ESNumber>(out["WindowHeight"]);
    }
    return config.config_para;
}
void LastUsedSettings::CreateCommonSettingsFile(Configuration& config)
{
    std::string common_settings_file = this->SettingsFilePath.c_str();
    common_settings_file = common_settings_file + "CommonSettings.SF2";
    if(this->CheckCommonSettingsFile()){
        this->DeleteCommonSettingsFile();
    }
    ESDictionary dict;
    dict["ESUnit"] = (ESNumber)config.config_para.units;
    dict["ESLastUsedSettings"] = (ESNumber)config.config_para.last_used_settings;
    dict["ESCompression"] = (ESNumber)config.config_para.compression;
    dict["ESOpenFolder"] = (ESNumber)config.config_para.open_img_folder;
    dict["ESSleepTimer"] = (ESNumber)config.config_para.sleep_timer;
    dict["ESACPower"] = (ESNumber)config.config_para.ac_power;
    dict["ESUSBPower"] = (ESNumber)config.config_para.usb_power;
    dict["WindowPositionX"] = (ESNumber)config.config_para.WindowPositionX;
    dict["WindowPositionY"] = (ESNumber)config.config_para.WindowPositionY;
    dict["WindowWidth"] = (ESNumber)config.config_para.WindowWidth;
    dict["WindowHeight"] = (ESNumber)config.config_para.WindowHeight;
    ES_CMN_FUNCS::JSON::DictionaryToJSONFile(dict, common_settings_file);
}
void LastUsedSettings::DeleteCommonSettingsFile()
{
    std::string common_settings_file = this->SettingsFilePath.c_str();
    common_settings_file = common_settings_file + "CommonSettings.SF2";
    try {
        DeleteFile(common_settings_file.c_str());
    }
    catch (...) {
        throw;
    }
}
bool LastUsedSettings::CreateSettingsFile(std::string model_id, const SCANPARA device_data)
{
    std::string model_id_folder = this->SettingsFilePath.c_str();
    model_id_folder = model_id_folder + model_id;
    std::string user_settings_file = model_id_folder + "/UserSettings.SF2";
    std::string dir1(qPrintable(QStandardPaths::writableLocation(QStandardPaths::HomeLocation)));
    ES_CMN_FUNCS::PATH::ES_CombinePath(dir1, dir1,"/.epsonscan2/");
    if(!(bool)ES_CMN_FUNCS::PATH::ES_IsExistFile(dir1)){
        ES_CMN_FUNCS::PATH::ES_MakeFolder(dir1);
    }
    if(ES_CMN_FUNCS::PATH::ES_IsExistFolder(this->SettingsFilePath) == FALSE)
    {
        ES_CMN_FUNCS::PATH::ES_MakeFolder(this->SettingsFilePath);
    }
    bool result = (bool)ES_CMN_FUNCS::PATH::ES_IsExistFile(model_id_folder);
    if ( !result ) {
        result = ES_CMN_FUNCS::PATH::ES_MakeFolder(model_id_folder);
        if ( !result ) {
            return false;
        }
    }
    result =ES_CMN_FUNCS::PATH::ES_IsExistFile(user_settings_file);
    if ( !result ) {
       std::ofstream(user_settings_file.c_str());
       std::fstream SettingFile(user_settings_file.c_str());
       if (SettingFile.fail())
       {
           return false;
       }
    }
    this->CreateSettingsDict(user_settings_file, device_data);
    return true;
}
void LastUsedSettings::CreateSettingsDict(std::string user_settngs_file, const SCANPARA device_data)
{
    ESDictionary dict;
    ESDictionary preset;
    ESDictionary devnum;
    dict["FunctionalUnit"] = (ESNumber)device_data.FunctionalUnit.select;
    dict["FunctionalUnit_Auto"] = (ESNumber)device_data.FunctionalUnit_Auto;
    dict["ColorType"] = (ESNumber)device_data.ColorType.select;
    dict["Resolution"] = (ESNumber)device_data.Resolution.select;
    dict["ImageFormat"] = (ESNumber)device_data.ImageFormatSettings.ImageFormat.select;
    dict["Gamma"] = (ESNumber)device_data.Gamma.select;
    dict["ScanAreaWidth"] = (ESNumber)device_data.ScanArea.ScanAreaWidth.select;
    dict["ScanAreaHeight"] = (ESNumber)device_data.ScanArea.ScanAreaHeight.select;
    dict["ScanAreaOffsetX"] = (ESNumber)device_data.ScanArea.ScanAreaOffsetX.select;
    dict["ScanAreaOffsetY"] = (ESNumber)device_data.ScanArea.ScanAreaOffsetY.select;
    dict["FixedDocumentSize"] = (ESNumber)device_data.ScanArea.FixedSize;
    int temp_num = device_data.ScanArea.UserDefine_Width * 100;
    dict["UserScanAreaWidth"] = (ESNumber)temp_num;
    temp_num = device_data.ScanArea.UserDefine_Length * 100;
    dict["UserScanAreaHeight"] = (ESNumber)temp_num;
    dict["DropoutColor"] = (ESNumber)device_data.DropoutColor.select;
    dict["Orientation"] = (ESNumber)device_data.Orientation.select;
    dict["MultiTiffEnabled"] = (ESNumber)device_data.ImageFormatSettings.MultiTiffEnabled.select;
    dict["MultiTiffCompression"] = (ESNumber)device_data.ImageFormatSettings.TiffCompression.select;
    dict["Contrast"] = (ESNumber)device_data.Contrast.select;
    dict["Brightness"] = (ESNumber)device_data.Brightness.select;
    dict["Threshold"] = (ESNumber)device_data.Threshold.select;
    dict["JpegQuality"] = (ESNumber)device_data.ImageFormatSettings.JpegQuality.select;
    dict["jpegProgressive"] = (ESNumber)device_data.ImageFormatSettings.JpegProgressive.select;
    dict["DuplexType"] = (ESNumber)device_data.DuplexType.select;
    dict["PagesTobeScanned"] = (ESNumber)device_data.PagesTobeScanned.select;
    dict["DoubleFeedDetectionLevel"] = (ESNumber)device_data.DoubleFeedDetectionValue.DoubleFeedDetectionLevel.select;
    dict["DoubleFeedDetection"] = (ESNumber)device_data.DoubleFeedDetectionValue.DoubleFeedDetection.select;
    dict["DoubleFeedDetectionAreaMin"] = (ESNumber)device_data.DoubleFeedDetectionValue.DoubleFeedDetectionAreaMin.select;
    dict["DoubleFeedDetectionAreaLength"] = (ESNumber)device_data.DoubleFeedDetectionValue.DoubleFeedDetectionAreaLength.select;
    dict["BlankPageSkip"] = (ESNumber)device_data.BlankPageSkipKey.select;
    dict["BlankPageSkipLevel"] = (ESNumber)device_data.BlankPageSkipLevelKey.select;
    dict["BackgroundRemoval"] = (ESNumber)device_data.BackgroundRemoval.select;
    dict["PaperDeskew"] = (ESNumber)device_data.PaperDeskew.select;
    dict["AutoSize"] = (ESNumber)device_data.ScanArea.AutoSize.select;
    dict["PaperEndDetection"] = (ESNumber)device_data.ScanArea.PaperEndDetection.select;
    dict["PaperEndDetectionMemory"] = (ESNumber)device_data.ScanArea.PaperEndDetectionMemory;
    dict["TransferCompression"] = (ESNumber)device_data.TransferCompression.select;
    dict["AddPages"] = (ESNumber)device_data.Add_Pages;
    dict["FileNamePrefix"] = (ESString)device_data.Prefix;
    dict["PDFAllPages"] = (ESNumber)device_data.ImageFormatSettings.PDFImgFormatOption.AllPage;
    dict["PDFSelectPage"] = (ESNumber)device_data.ImageFormatSettings.PDFImgFormatOption.page;
    dict["UserDefinePath"] = (ESString)device_data.UserDefinePath;
    dict["Folder"] = (ESNumber)device_data.Folder;
    dict["ImageOption"] = (ESNumber)device_data.ImageOption;
    dict["DNShow_PlasticCard_dialog"] = (ESNumber)device_data.DNShow_PlasticCard_dialog;
    dict["DNShow_LongPaperWarning_dialog"] = (ESNumber)device_data.DNShow_LongPaperWarning_dialog;
    dict["OverDurationAlert"] = (ESNumber)device_data.OverDurationAlert;
    dict["NearDurationAlert"] = (ESNumber)device_data.NearDurationAlert;
    dict["AFMMode"] = (ESNumber)device_data.AutomaticFeedingMode.select;
    dict["AFMTimeout"] = (ESNumber)device_data.AFMTimeout.select;
    dict["BackgroundColor"] = (ESNumber)device_data.BackgroundColor.select;
    devnum["0"] = dict;
    preset["Preset"] = devnum;
    ES_CMN_FUNCS::JSON::DictionaryToJSONFile(preset, user_settngs_file);
}
bool LastUsedSettings::ReadSettingsFile(std::string model_id, SCANPARA* device_data)
{
    std::string user_settings_file = this->SettingsFilePath.c_str();
    user_settings_file = user_settings_file + model_id + "/UserSettings.SF2";
    const bool result = ES_CMN_FUNCS::PATH::ES_IsExistFolder(user_settings_file);
    if ( !result ) {
        return false;
    }
    this->SetDeviceData(user_settings_file, device_data);
    return true;
}
bool LastUsedSettings::ReadCmdSettingsFile(std::string settings_file_path, SCANPARA* device_data)
{
    const bool result = ES_CMN_FUNCS::PATH::ES_IsExistFolder(settings_file_path);
    if ( !result ) {
        return false;
    }
    this->SetDeviceData(settings_file_path, device_data);
    return true;
}
#include "ESAnyCastUtils.h"
void LastUsedSettings::SetDeviceData(std::string settings_file_path, SCANPARA* device_data)
{
    ESDictionary out;
    ES_CMN_FUNCS::JSON::JSONFiletoDictionary(settings_file_path, out);
    ESDictionary preset = SAFE_ANY_VALUE_FROM_DICT(out,"Preset",ESDictionary, ESDictionary());
    ESDictionary devnum = SAFE_ANY_VALUE_FROM_DICT(preset,"0",ESDictionary, ESDictionary());
    device_data->FunctionalUnit.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"FunctionalUnit",ESNumber,ADF);
    device_data->FunctionalUnit_Auto = SAFE_ANY_VALUE_FROM_DICT(devnum,"FunctionalUnit_Auto",ESNumber,0);
    device_data->ColorType.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"ColorType",ESNumber,ColorTypeRGB24);
    device_data->Resolution.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"Resolution",ESNumber,200);
    device_data->ImageFormatSettings.ImageFormat.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"ImageFormat",ESNumber,ImageFormatColorJpegMonoRaw);
    device_data->Gamma.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"Gamma",ESNumber,22);
    device_data->ScanArea.ScanAreaWidth.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"ScanAreaWidth",ESNumber,0);
    device_data->ScanArea.ScanAreaHeight.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"ScanAreaHeight",ESNumber,0);
    device_data->ScanArea.ScanAreaOffsetX.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"ScanAreaOffsetX",ESNumber,0);
    device_data->ScanArea.ScanAreaOffsetY.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"ScanAreaOffsetY",ESNumber,0);
    device_data->ScanArea.FixedSize = (DocumentSize)SAFE_ANY_VALUE_FROM_DICT(devnum,"FixedDocumentSize",ESNumber,Auto_Detect);
    device_data->ScanArea.UserDefine_Width = SAFE_ANY_VALUE_FROM_DICT(devnum,"UserScanAreaWidth",ESNumber,0);
    device_data->ScanArea.UserDefine_Width /= 100;
    device_data->ScanArea.UserDefine_Length = SAFE_ANY_VALUE_FROM_DICT(devnum,"UserScanAreaHeight",ESNumber,0);
    device_data->ScanArea.UserDefine_Length /= 100;
    device_data->DropoutColor.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"DropoutColor",ESNumber,DropoutColorNone);
    device_data->Orientation.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"Orientation",ESNumber,OrientationNone);
    device_data->ImageFormatSettings.MultiTiffEnabled.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"MultiTiffEnabled",ESNumber,0);
    device_data->ImageFormatSettings.TiffCompression.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"MultiTiffCompression",ESNumber,TiffCompressionNone);
    device_data->Contrast.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"Contrast",ESNumber,0);
    device_data->Brightness.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"Brightness",ESNumber,0);
    device_data->Threshold.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"Threshold",ESNumber,110);
    device_data->ImageFormatSettings.JpegQuality.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"JpegQuality",ESNumber,85);
    device_data->ImageFormatSettings.JpegProgressive.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"jpegProgressive",ESNumber,false);
    device_data->DuplexType.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"DuplexType",ESNumber,Double_Sided);
    device_data->PagesTobeScanned.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"PagesTobeScanned",ESNumber,0);
    device_data->DoubleFeedDetectionValue.DoubleFeedDetectionLevel.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"DoubleFeedDetectionLevel",ESNumber,DoubleFeedDetectionLow);
    device_data->DoubleFeedDetectionValue.DoubleFeedDetection.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"DoubleFeedDetection",ESNumber,true);
    device_data->DoubleFeedDetectionValue.DoubleFeedDetectionAreaMin.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"DoubleFeedDetectionAreaMin",ESNumber,0);
    device_data->DoubleFeedDetectionValue.DoubleFeedDetectionAreaLength.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"DoubleFeedDetectionAreaLength",ESNumber,0);
    device_data->BlankPageSkipKey.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"BlankPageSkip",ESNumber,false);
    device_data->BlankPageSkipLevelKey.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"BlankPageSkipLevel",ESNumber,10);
    device_data->BackgroundRemoval.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"BackgroundRemoval",ESNumber,BackgroundRemovalNone);
    device_data->PaperDeskew.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"PaperDeskew",ESNumber,true);
    device_data->ScanArea.AutoSize.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"AutoSize",ESNumber,true);
    device_data->ScanArea.PaperEndDetection.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"PaperEndDetection",ESNumber,false);
    device_data->ScanArea.PaperEndDetectionMemory = SAFE_ANY_VALUE_FROM_DICT(devnum,"PaperEndDetectionMemory",ESNumber,false);
    device_data->TransferCompression.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"TransferCompression",ESNumber,true);
    device_data->Add_Pages = SAFE_ANY_VALUE_FROM_DICT(devnum,"AddPages",ESNumber,false);
    device_data->Prefix = SAFE_ANY_VALUE_FROM_DICT(devnum,"FileNamePrefix",ESString,"img");
    device_data->ImageFormatSettings.PDFImgFormatOption.AllPage = SAFE_ANY_VALUE_FROM_DICT(devnum,"PDFAllPages",ESNumber,true);
    if(device_data->ImageFormatSettings.PDFImgFormatOption.AllPage){
        device_data->ImageFormatSettings.PDFImgFormatOption.page = SAFE_ANY_VALUE_FROM_DICT(devnum,"PDFSelectPage",ESNumber,0);
    }else {
        device_data->ImageFormatSettings.PDFImgFormatOption.page = SAFE_ANY_VALUE_FROM_DICT(devnum,"PDFSelectPage",ESNumber,1);
    }
    device_data->UserDefinePath = SAFE_ANY_VALUE_FROM_DICT(devnum,"UserDefinePath",ESString,qPrintable(QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation)));
    device_data->Folder = SAFE_ANY_VALUE_FROM_DICT(devnum,"Folder",ESNumber,Document);
    device_data->ImageOption = SAFE_ANY_VALUE_FROM_DICT(devnum,"ImageOption",ESNumber,ImageOptionNone);
    device_data->DNShow_PlasticCard_dialog = SAFE_ANY_VALUE_FROM_DICT(devnum,"DNShow_PlasticCard_dialog",ESNumber,false);
    device_data->DNShow_LongPaperWarning_dialog = SAFE_ANY_VALUE_FROM_DICT(devnum,"DNShow_PlasticCard_dialog",ESNumber,false);
    device_data->OverDurationAlert = SAFE_ANY_VALUE_FROM_DICT(devnum,"OverDurationAlert",ESNumber,false);
    device_data->NearDurationAlert = SAFE_ANY_VALUE_FROM_DICT(devnum,"NearDurationAlert",ESNumber,false);
    device_data->AutomaticFeedingMode.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"AFMMode",ESNumber,false);
    device_data->AFMTimeout.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"AFMTimeout",ESNumber,180);
    device_data->BackgroundColor.select = SAFE_ANY_VALUE_FROM_DICT(devnum,"BackgroundColor",ESNumber,BackgroundColorBlack);
}
